#!/usr/bin/env python3

import argparse
import logging
import os
from utils.path_handler import PathHandler
from utils.querier import Querier,RemoteObject
from utils.uploader import Uploader
from utils.releaser import Releaser
from utils.synchronizer import Synchronizer

HelpMessage="""
"""

logging.basicConfig(format='%(levelname)s [GridMap Binary Release]: %(message)s',level=logging.WARN)

config_file = os.path.dirname(os.path.realpath(os.path.abspath(__file__)))+"/map_binary_release.json"
print(config_file)
if not os.getenv("AD_RELEASE_MAP_PATH"):
  os.environ["AD_RELEASE_MAP_PATH"] = "/opt/saffron/map"

def parse():
    parser = argparse.ArgumentParser(description=HelpMessage)
    subparsers = parser.add_subparsers(help='sub-command help')

    query_subparser = subparsers.add_parser('query', help="query")
    query_subparser.set_defaults(func=query)

    release_subparser = subparsers.add_parser('release', help="release")
    release_subparser.set_defaults(func=release)

    synchronize_subparser = subparsers.add_parser('synchronize', help="synchronize")
    synchronize_subparser.set_defaults(func=synchronize)

    upload_subparser = subparsers.add_parser('upload', help="upload")
    upload_subparser.set_defaults(func=upload)

    return parser.parse_args()

def query(args):
    querier = Querier(config_file)
    for remote_object_list in querier.remote_objects.values():
        for remote_object in remote_object_list:
            print(remote_object.name + "  "+ remote_object.mode + "  " + remote_object.remote_path)

def release(args):
    releaser = Releaser(config_file)
    releaser.release_all_object()

def synchronize(args):
    synchronizer = Synchronizer(config_file)
    synchronizer.sync()

def upload(args):
    uploader = Uploader(config_file)
    uploader.upload_all_objects()

if __name__ == "__main__":
    logging.info("Binary release interface for map")
    args = parse()
    args.func(args)
