#!/usr/bin/env python3

import json
import logging
import os

class ModeConfig(object):
    
    def __init__(self):
        self.mode = None
        self.remote_location = None
        self.local_location = None
        self.use_sha256 = False

    def from_dict(self, my_dict):
        if "mode" in my_dict:
            self.mode = my_dict["mode"]
        else:
            logging.error("\033[1;31m[ERROR]\033[0mWrong config: no 'mode'")
            return 1
        if "remote_location" in my_dict:
            self.remote_location = my_dict["remote_location"]
        else:
            logging.error("\033[1;31m[ERROR]\033[0mWrong config: no 'remote_location'")
            return 1
        if "local_location" in my_dict:
            self.local_location = my_dict["local_location"]
        else:
            logging.error("\033[1;31m[ERROR]\033[0mWrong config: no 'local_location'")
            return 1
        if "use_sha256" in my_dict:
            self.use_sha256 = my_dict["use_sha256"]
        return 0

class ConfigReader(object):

    def __init__(self):
        self.manifest_file = None
        self.project = None
        self.modes = dict()
        self.use_symblic_link = None
        self.symbolic_dest = None

    def read(self, config_file):
        if not os.path.exists(config_file):
            logging.error("\033[1;31m[ERROR]\033[0mConfig file not exits.")
            return 1
        with open(config_file) as file:
            raw_data = json.load(file)
        if not isinstance(raw_data, dict):
            logging.error("\033[1;31m[ERROR]\033[0mWrong config format.")
            return 1
        try:
            self.project = raw_data["project"]
            self.manifest_file = raw_data["manifest_file"]
            self.use_symblic_link = raw_data["symbolic_link"]["use"]
            self.symbolic_dest = raw_data["symbolic_link"]["destination"]
            self.commitid_reference_git = raw_data["commitid_reference_git"]
            modes = raw_data["modes"]
            for mode in modes:
                self.modes[mode["mode"]] = ModeConfig()
                self.modes[mode["mode"]].from_dict(mode)
        except:
            logging.error("Wrong config")

if __name__ == "__main__":
    pass