#!/usr/bin/env python3

import sys
import os
import logging
import re
from .config_reader import ModeConfig

class PathHandler(object):

    def __init__(self):
        self.resource_path = ""
        script_path = os.path.realpath(os.path.abspath(sys.argv[0]))
        tmp_index = 0
        path_fragments = list(script_path.split('/'))
        for i in range(len(path_fragments)):
            if path_fragments[i] == "resource":
                tmp_index = i
        if tmp_index == 0:
            logging.error("\033[1;31m[ERROR]\033[0m Not in saffron. Currently not support.")
            sys.exit(1)
        else:
            for i in range(tmp_index + 1):
                self.resource_path += ("/" + path_fragments[i])
        logging.info("Path to resource : {}".format(self.resource_path))

    def ReplaceResourcePath(self, path):
        return path.replace("$MAP_RESOURCE_ROOT", self.resource_path)

    def ReplaceEnv(self, path):
        a = re.match("\$ENV{[\-A-Za-z_]*}", path)
        while a:
            env = str(path[a.span()[0]:a.span()[1]]).lstrip('$ENV{').rstrip('}')
            rep = os.environ[env]
            if not rep:
                logging.error("\033[1;31m[ERROR]\033[0m environmental variable {} not found".format(env))
            path = path.replace(str(path[a.span()[0]:a.span()[1]]),rep)
            a = re.search("\$ENV{[-A-Za-z_]*}", path)
        return path

    def ReplaceConfigInfo(self, path, config):
        a = re.search("<[A-Za-z_-]*>", path)
        while a:
            att = a[0].lstrip('<').rstrip('>')
            rep = config.get(att)
            if not rep:
                logging.error("\033[1;31m[ERROR]\033[0m config attribute {} not found".format(att))
            path = path.replace(a[0],rep)
            a = re.search("<[A-Za-z_-]*>", path)
        return path

    def ConfigurePath(self, path):
        path = self.ReplaceEnv(path)
        path = self.ReplaceResourcePath(path)
        # path = self.ReplaceConfigInfo(path, config)
        return self.RemoveDuplicateSlash(path)

    def GetDir(self, path):
        if self.IsDir(path):
            return path
        else:
            last_slash = path.rfind("/")
            if last_slash != -1:
                return path[:last_slash+1]
            else:
                return path

    def GetFile(self, path):
        if self.IsDir(path):
            return None
        else:
            last_slash = path.rfind("/")
            if last_slash != -1:
                return path[last_slash+1:]
            else:
                return path

    def IsDir(self, path):
        if path[-1] == '/':
            return True
        else:
            return False

    def RemoveDuplicateSlash(self, path):
        return path.replace("//","/")
