#!/usr/bin/env python3

"""
provide Uploader for binary release file management.
Uploader class is in charge of :
   parse binary release config manifest
   upload test objects to AWS
REMIND: one needs to commit manifest.xml before running this scripts
"""

import sys
import logging
import os
from .manifest_parser_and_generator import Object, Manifest
from .querier import Querier
from .path_handler import PathHandler
from .config_reader import ConfigReader

class Uploader(object):

    def __init__(self, config_file):
        self.querier = Querier(config_file)
        self.path_handler = PathHandler()
        self.resource_path = self.path_handler.resource_path
        self.config = ConfigReader()
        self.config.read(config_file)
        self.commitid = os.popen("git --git-dir={}/.git rev-parse --short HEAD"
                        .format(self.path_handler.ConfigurePath(self.config.commitid_reference_git))).readline().strip('\n')
        self.local_test_location = self.path_handler.ConfigurePath(self.config.modes["test"].local_location)

        self.manifest = Manifest()
        manifest_file = self.path_handler.ConfigurePath(self.config.manifest_file)
        if not os.path.exists(manifest_file):
            logging.error("\033[1;31m[ERROR]\033[0mManifest file {} not found!".format(manifest_file))
            exit(1)
        self.manifest.parse_from_file(manifest_file)

    def upload_all_objects(self, force=False):
        if not self.manifest.is_valid():
            logging.error("\033[1;31m[ERROR]\033[0m Please check your manifest.")
            return False
        flag_success = True
        for local_objects in self.manifest.objects.values():
            for local_object in local_objects:
                if local_object.mode == "test":
                    if local_object.form == "folder":
                        # remove tmp file
                        if os.path.exists(self.local_test_location + "/tmp.tar"):
                            ret = os.system("rm {}".format(self.local_test_location + "/tmp.tar"))
                            if 0 != ret:
                                os.system("sudo rm {}".format(self.local_test_location + "/tmp.tar"))
                        # check if folder is empty
                        if len(os.listdir(os.path.join(self.local_test_location, local_object.name))) == 0:
                            logging.warn("test folder object {} is empty, will skip".format(local_object.name))
                            continue
                        ret = os.system("tar -cf {}/tmp.tar -C {} {}"
                            .format(self.local_test_location, self.local_test_location, local_object.name))
                        if 0 != ret:
                            ret = os.system("sudo tar -cf {}/tmp.tar -C {} {}"
                                .format(self.local_test_location, self.local_test_location, local_object.name))
                        local_object_path = self.local_test_location + "/tmp.tar"
                        remote_object_name = local_object.name + "." + self.commitid
                    else:
                        local_object_path = self.local_test_location + "/" + local_object.name
                        remote_object_name = local_object.name + "." + self.commitid

                    if not os.path.exists(local_object_path):
                        logging.error("local object {} not found!".format(local_object_path))
                        continue
                    remote_object_name = local_object.name + "." + self.commitid
                    ret = self.querier.upload_object(local_object_path, remote_object_name)
                    flag_success = flag_success and not ret
                    # remove tmp file
                    if os.path.exists(self.local_test_location + "/tmp.tar"):
                        ret = os.system("rm {}".format(self.local_test_location + "/tmp.tar"))
                        if 0 != ret:
                            os.system("sudo rm {}".format(self.local_test_location + "/tmp.tar"))
                else:
                    logging.info("[INFO] Skip object of release mode.")
        if flag_success:
            print("\033[1;32m[{} BINARY RELEASE MANAGER] UPLOAD SUCCESS\033[0m".format(self.config.project))
        else:
            print("\033[1;31m[{} BINARY RELEASE MANAGER] UPLOAD FAILED\033[0m".format(self.config.project), file=sys.stderr)

if __name__ == "__main__":
    config_file = ""
    uploader = Uploader(config_file)
    uploader.upload_all_objects()
