#pragma once

#include <iostream>
#include <string>

#include "map/common/status.hpp"

#include "ad_log/ad_log.hpp"

namespace saffron {
namespace hdm {

inline std::string ADGetStatusString(mapStatus_t error) {
    switch (error) {
        case MAP_SUCCESS:
            return "Success";
        case MAP_INVALID_PARAM:
            return "Invalid arguments";
        case MAP_FILE_NOT_FOUND:
            return "File not found";
        case MAP_NULL_PTR:
            return "null pointer";
        case MAP_END_OF_FILE:
            return "reach end of file";
        case ROUTING_ERROR:
            return "routing error";
        case MAP_ERROR:
            return "map error";
        case MAP_NO_INIT_ERROR:
            return "no init error";
        default:
            return "";
    }
}

#define PRINT_AD_ERROR_MESSAGE(msg) LOGP_ERROR("{}", msg)

/// 3 in 1 Checker
#define __GET_ARG4(arg1, arg2, arg3, arg4, ...) arg4
#define MAP_CHECK_SUCCESS(err)                                              \
    {                                                                       \
        saffron::hdm::mapStatus_t pst = err;                                \
        if (saffron::hdm::MAP_SUCCESS != pst) {                             \
            PRINT_AD_ERROR_MESSAGE(ADGetStatusString(pst)); \
            return pst;                                                     \
        }                                                                   \
    }

#define MAP_CHECK_ERRORCODE(err, err_code)                                  \
    {                                                                       \
        saffron::hdm::mapStatus_t pst = err;                                \
        saffron::hdm::mapStatus_t _err_code = err_code;                     \
        if (_err_code != pst) {                                             \
            PRINT_AD_ERROR_MESSAGE(ADGetStatusString(pst)); \
            return pst;                                                     \
        }                                                                   \
    }
#define MAP_CHECK_ERROR_WITH_MESSAGE(err, err_code, msg)              \
    {                                                                 \
        saffron::hdm::mapStatus_t pst = err;                          \
        saffron::hdm::mapStatus_t _err_code = err_code;               \
        if (_err_code != pst) {                                       \
            LOGP_ERROR("{} {}", ADGetStatusString(pst), msg); \
            return pst;                                               \
        }                                                             \
    }

#define MAP_CHECK_ERROR(...)                                                   \
    __GET_ARG4(__VA_ARGS__, MAP_CHECK_ERROR_WITH_MESSAGE, MAP_CHECK_ERRORCODE, \
               MAP_CHECK_SUCCESS)                                              \
    (__VA_ARGS__)  /// end 3 in 1 checker

#define MAP_ASSERTION_TRUE_WITHOUT_MESSAGE(res, err)                        \
    {                                                                       \
        saffron::hdm::mapStatus_t pst = err;                                \
        if (!res) {                                                         \
            PRINT_AD_ERROR_MESSAGE(ADGetStatusString(pst)); \
            return pst;                                                     \
        }                                                                   \
    }

#define MAP_ASSERTION_TRUE_WITH_MESSAGE(res, err, msg)                \
    {                                                                 \
        saffron::hdm::mapStatus_t pst = err;                          \
        if (!res) {                                                   \
            LOGP_ERROR("{} {}", ADGetStatusString(pst), msg); \
            return pst;                                               \
        }                                                             \
    }

#define MAP_ASSERTION_TRUE(res, err, ...)                                \
    __GET_ARG4(res, err, ##__VA_ARGS__, MAP_ASSERTION_TRUE_WITH_MESSAGE, \
               MAP_ASSERTION_TRUE_WITHOUT_MESSAGE)                       \
    (res, err, ##__VA_ARGS__)

#define MAP_CHECK_FILE_READABLE(FILE_NAME, ERR)                     \
    do {                                                            \
        if (access(FILE_NAME, R_OK | F_OK) != 0) {                  \
            LOGP_ERROR(                                             \
                "Failed to access file \" {}"                       \
                " \"! Please make sure it's existed and readable.", \
                FILE_NAME);                                         \
            return saffron::MAP_FILE_NOT_FOUND;                     \
        }                                                           \
    } while (0)

}  // namespace hdm
}  // namespace saffron
