#pragma once

#include <utility>
#include <vector>
#include <string>

#include "ad_common/data_type/base.hpp"
namespace saffron {
namespace routing {

struct RoutingRequest {
    PointLLH_t departure{0, 0, 0};
    std::vector<PointLLH_t> mid_points;
    std::vector<PointLLH_t> black_list_points;
    std::vector<std::string> mid_lane_ids;
    std::vector<std::string> black_list_lanes;
    PointLLH_t destination{0, 0, 0};
    bool pullover = true;
};

struct Connection {
    std::string from_section;
    std::string to_section;
    std::string from_lane;
    std::string to_lane;

    Connection() = default;

    Connection(std::string fr, std::string tr, std::string fl, std::string tl)
        : from_section(std::move(fr)),
          to_section(std::move(tr)),
          from_lane(std::move(fl)),
          to_lane(std::move(tl)) {}

    Connection &operator=(const Connection &con) = default;
    bool operator==(const Connection &con) const {
        return from_section == con.from_section &&
               to_section == con.to_section && from_lane == con.from_lane &&
               to_lane == con.to_lane;
    }
};

enum class NodeType { ROAD, JUNCTION };

struct RoutingNode {
    std::string id;  // section id or junction id
    NodeType type;
    std::string lane_id;
    std::string road_id;
    double accu_length;
    Connection connection;

    bool operator==(const RoutingNode &rn) const {
        return id == rn.id && type == rn.type && lane_id == rn.lane_id &&
               connection == rn.connection;
    }
};

enum RoutingResponseStatus {
    AD_ROUTING_SUCCESS = 0,
    AD_ROUTING_FAILED = 1,
};

struct RoutingResponse {
    uint64_t timestamp;
    RoutingResponseStatus status;
    std::vector<RoutingNode> node_list;
    std::vector<PointLLH_t> navigation_line;
    PointENU_t destination;
};

}  // namespace routing
}  // namespace saffron
