#pragma once
#include <Eigen/Core>
#include "ad_common/data_type/base.hpp"

namespace saffron {
namespace hdm {
constexpr double kMathEpsilon = 1e-10;

float64_t CrossProd2d(const Eigen::Vector2d& v1, const Eigen::Vector2d& v2);
float64_t CrossProd2d(const Eigen::Vector2d& start_point,
                      const Eigen::Vector2d& end_point1,
                      const Eigen::Vector2d& end_point2);
float64_t InnerProd2d(const Eigen::Vector2d& v1, const Eigen::Vector2d& v2);

float64_t Distance2d(const Eigen::Vector2d& p1, const Eigen::Vector2d& p2);

float64_t Square(double x);
float64_t CrossProd2d(const Point3D_t& v1, const Point3D_t& v2);
float64_t InnerProd2d(const Point3D_t& v1, const Point3D_t& v2);

float64_t Distance2d(const Point3D_t& p1, const Point3D_t& p2);

Point3D_t Minus(const Point3D_t& p1, const Point3D_t& p2);
Point3D_t Plus(const Point3D_t& p1, const Point3D_t& p2);
Point3D_t Divide(const Point3D_t& p, float64_t denominator);
Point3D_t Multiply(const Point3D_t& p, float64_t ratio);
float64_t Norm(const Point3D_t& p);
bool Equal(const Point3D_t& p1, const Point3D_t& p2);
bool Equal(const PointLLH_t& p1, const PointLLH_t& p2);
template <typename T1, typename T2>
T2 Transform2VehicleCoord2D(const T1& vehicle_pt,
                           float64_t vehicle_heading,
                           const T2& src_pt) {
    T2 dst_pt;
    auto cos_heading = cos(vehicle_heading);
    auto sin_heading = sin(vehicle_heading);
    auto x = src_pt.x - vehicle_pt.x;
    auto y = src_pt.y - vehicle_pt.y;

    dst_pt.x = x * cos_heading + y * sin_heading;
    dst_pt.y = y * cos_heading - x * sin_heading;
    return dst_pt;
}
template <typename T1, typename T2>
T2 Transform2WorldCoord2D(const T1& vehicle_pt,
                         float64_t vehicle_heading,
                         const T2& src_pt) {
    T2 dst_pt;
    auto cos_heading = cos(vehicle_heading);
    auto sin_heading = sin(vehicle_heading);

    dst_pt.x = src_pt.x * cos_heading - src_pt.y * sin_heading + vehicle_pt.x;
    dst_pt.y = src_pt.y * cos_heading + src_pt.x * sin_heading + vehicle_pt.y;
    return dst_pt;
}
}  // namespace hdm
}  // namespace saffron
