#pragma once
#include <string>
#include <vector>
#include <Eigen/Core>
#include "map/math_tools/math_util.hpp"

namespace saffron {
namespace hdm {
class Polygon2d {
 public:
    Polygon2d() = default;

    explicit Polygon2d(std::vector<Eigen::Vector2d> points);

    float64_t DistanceTo(const Eigen::Vector2d &point) const;
    float64_t HeadingFrom(const Eigen::Vector2d &point) const;
    bool IsPointIn(const Eigen::Vector2d &point) const;

    bool IsPointOnBoundary(const Eigen::Vector2d &point) const;

    inline float64_t MinX() const { return min_x_; }
    inline float64_t MaxX() const { return max_x_; }
    inline float64_t MinY() const { return min_y_; }
    inline float64_t MaxY() const { return max_y_; }
    inline float64_t CenterX() const { return center_x_; }
    inline float64_t CenterY() const { return center_y_; }
    inline const std::vector<Eigen::Vector2d> &GetBoundary() const {
        return points_;
    }

 private:
    void ComputeCentroid();
    bool IsWithIn(double val, double bound1, double bound2) const;
    size_t Next(size_t at) const;
    std::vector<Eigen::Vector2d> points_;
    std::vector<float64_t> line_segment_length_;
    std::vector<Eigen::Vector2d> unit_direction_;
    uint32_t point_num_ = 0;
    float64_t min_x_ = 0.0;
    float64_t max_x_ = 0.0;
    float64_t min_y_ = 0.0;
    float64_t max_y_ = 0.0;
    float64_t center_x_ = 0.0;
    float64_t center_y_ = 0.0;
};
}  // namespace hdm
}  // namespace saffron
