#pragma once
#include <string>
#include <vector>
#include <algorithm>
#include <limits>
#include <Eigen/Core>
#include "map/math_tools/math_util.hpp"

namespace saffron {
namespace hdm {
class Polyline2d {
 public:
    Polyline2d() = default;

    explicit Polyline2d(std::vector<Eigen::Vector2d> points);

    double DistanceTo(const Eigen::Vector2d &point) const;
    float64_t HeadingFrom(const Eigen::Vector2d &point) const;
    bool IsPointIn(const Eigen::Vector2d &point) const;
    inline float64_t MinX() const { return min_x_; }
    inline float64_t MaxX() const { return max_x_; }
    inline float64_t MinY() const { return min_y_; }
    inline float64_t MaxY() const { return max_y_; }
    inline float64_t CenterX() const { return center_x_; }
    inline float64_t CenterY() const { return center_y_; }

 private:
    bool IsWithIn(float64_t val, float64_t bound1, float64_t bound2) const;
    std::vector<Eigen::Vector2d> points_;
    std::vector<float64_t> line_segment_length_;   // size = point_.size() - 1;
    std::vector<Eigen::Vector2d> unit_direction_;  // size = point_.size() -1 ;
    uint32_t point_num_ = 0;
    float64_t min_x_ = 0.0;
    float64_t max_x_ = 0.0;
    float64_t min_y_ = 0.0;
    float64_t max_y_ = 0.0;
    float64_t center_x_ = 0.0;
    float64_t center_y_ = 0.0;
};
}  // namespace hdm
}  // namespace saffron
