#pragma once
#include <string>
#include <vector>
#include <unordered_map>
#include <memory>
#include <limits>

#include "map/math_tools/box_kdtree.hpp"

namespace saffron {
namespace hdm {
template <class Object>
mapStatus_t TraverseSearchObjects(
    const PointENU_t& position,
    float64_t distance,
    const std::unordered_map<std::string, std::shared_ptr<Object>>&
        object_table,
    std::vector<std::shared_ptr<const Object>>* objects) {
    if (objects == nullptr) {
        MAP_CHECK_ERROR(MAP_NULL_PTR)
    }
    Eigen::Vector2d eigen_position(position.x, position.y);
    for (const auto& object : object_table) {
        if (object.second->GetPoly2d().DistanceTo(eigen_position) <= distance) {
            objects->push_back(object.second);
        }
    }
    return MAP_SUCCESS;
}

template <class Object>
mapStatus_t TraverseSearchNearestObjects(
    const PointENU_t& position,
    const std::unordered_map<std::string, std::shared_ptr<Object>>&
        object_table,
    std::shared_ptr<const Object>* nearest_object,
    float64_t* distance) {
    *distance = std::numeric_limits<float64_t>::max();
    if (nearest_object == nullptr || distance == nullptr) {
        MAP_CHECK_ERROR(MAP_NULL_PTR)
    }
    Eigen::Vector2d eigen_position(position.x, position.y);
    for (const auto& object : object_table) {
        auto dis = object.second->GetPoly2d().DistanceTo(eigen_position);
        if (dis < *distance) {
            *distance = dis;
            *nearest_object = object.second;
        }
    }
    return MAP_SUCCESS;
}
}  // namespace hdm
}  // namespace saffron
