#pragma once
#include <string>
#include <vector>
#include <memory>
#include <unordered_map>

#include "map/data_type/hdmap_data.hpp"
#include "map/math_tools/polygon2d.hpp"

namespace saffron {
namespace hdm {
class Junction {
 public:
    using ConstPtr = std::shared_ptr<const Junction>;
    using Ptr = std::shared_ptr<Junction>;
    using Table = std::unordered_map<std::string, Ptr>;
    using Vector = std::vector<ConstPtr>;

 public:
    Junction() = default;
    explicit Junction(JunctionData &&junction_data);
    explicit Junction(const JunctionData &junction_data);
    const JunctionData &GetData() const;
    const std::string &GetId() const;
    const std::vector<std::string> &GetOverlapIds() const;
    std::vector<std::string> GetSectionIds() const;
    std::vector<std::string> GetRoadIds() const;
    std::vector<std::string> GetLaneIds() const;
    const std::vector<Boundary> &GetBoundaries() const;
    const Polygon2d &GetPoly2d() const;
    bool IsPointIn(const Point3D_t &position) const;
    float64_t Distance2Point(const Point3D_t &position) const;
    float64_t HeadingFromPoint2Junction(const Point3D_t &position) const;

 private:
    void SetPolygon();

 private:
    JunctionData junction_data_;
    Polygon2d polygon_2d_;
};
}  // namespace hdm
}  // namespace saffron
