#pragma once
#include <string>
#include <vector>
#include <memory>
#include <unordered_map>
#include <limits>
#include <utility>

#include "map/data_type/hdmap_data.hpp"

namespace saffron {
namespace hdm {
/*************************** road ********************************/
class Road {
 public:
    using ConstPtr = std::shared_ptr<const Road>;
    using Ptr = std::shared_ptr<Road>;
    using Table = std::unordered_map<std::string, Ptr>;
    using Vector = std::vector<ConstPtr>;

 public:
    Road() = default;
    explicit Road(RoadData &&road_data);
    explicit Road(const RoadData &road_data);
    const RoadData &GetData() const;
    const std::string &GetId() const;
    const std::string &GetName() const;
    const float64_t &GetLength() const;
    const std::vector<std::string> &GetOverlapIds() const;
    const std::vector<std::string> &GetSectionIds() const;
    const std::vector<std::string> &GetPredecessorIds() const;
    const std::vector<std::string> &GetSuccessorIds() const;
    const RoadType &GetType() const;
    std::vector<std::string> GetJunctionIds() const;
    std::vector<std::string> GetStopLineIds() const;
    std::vector<std::string> GetGroupTLIds() const;

 private:
    RoadData road_data_;
};

/*************************** RoadLink ********************************/
class RoadLink {
 public:
    using ConstPtr = std::shared_ptr<const RoadLink>;
    using Ptr = std::shared_ptr<RoadLink>;
    using Table = std::unordered_map<std::string, Ptr>;
    using Vector = std::vector<ConstPtr>;

 public:
    RoadLink() = default;
    explicit RoadLink(RoadLinkData &&road_link_data);
    explicit RoadLink(const RoadLinkData &road_link_data);
    const RoadLinkData &GetData() const;
    const std::string &GetId() const;
    const std::string &GetFromRoadId() const;
    const std::string &GetToRoadId() const;
    TurnLink GetTurnLink() const;

 private:
    RoadLinkData road_link_data_;
};
}  // namespace hdm

}  // namespace saffron
