#pragma once
#include <string>
#include <vector>
#include <memory>
#include <unordered_map>
#include <limits>
#include <utility>

#include "map/data_type/hdmap_data.hpp"
#include "map/semantic_map/line.hpp"
#include "map/math_tools/polygon2d.hpp"

namespace saffron {
namespace hdm {
/*************************** Section ********************************/
class Section {
 public:
    using ConstPtr = std::shared_ptr<const Section>;
    using Ptr = std::shared_ptr<Section>;
    using Table = std::unordered_map<std::string, Ptr>;
    using Vector = std::vector<ConstPtr>;

 public:
    Section() = default;
    explicit Section(SectionData &&section_data);
    explicit Section(const SectionData &section_data);
    const SectionData &GetData() const;
    const std::string &GetId() const;
    const std::string &GetRoadId() const;
    const float64_t &GetLength() const;
    const std::vector<std::string> &GetOverlapIds() const;
    const std::vector<std::string> &GetLaneIds() const;
    const std::vector<std::string> &GetPredecessorIds() const;
    const std::vector<std::string> &GetSuccessorIds() const;
    const std::string &GetLeftBoundId() const;
    const std::string &GetRightBoundId() const;
    const Polygon2d &GetPoly2d() const;
    const Point3D_t &GetEndPoint() const;
    mapStatus_t SetPolygon(const Line::ConstPtr &left_line,
                           const Line::ConstPtr &right_line);
    bool IsPointIn(const Point3D_t &position) const;
    float64_t Distance2Point(const Point3D_t &position) const;
    std::vector<std::string> GetJunctionIds() const;
    std::vector<std::string> GetStopLineIds() const;
    std::vector<std::string> GetGroupTLIds() const;

 private:
    SectionData section_data_;
    Polygon2d polygon_2d_;
    Point3D_t end_point_{};
};

/*************************** SectionLink ********************************/
class SectionLink {
 public:
    using ConstPtr = std::shared_ptr<const SectionLink>;
    using Ptr = std::shared_ptr<SectionLink>;
    using Table = std::unordered_map<std::string, Ptr>;
    using Vector = std::vector<ConstPtr>;

 public:
    SectionLink() = default;
    explicit SectionLink(SectionLinkData &&section_link_data);
    explicit SectionLink(const SectionLinkData &section_link_data);
    const SectionLinkData &GetData() const;
    const std::string &GetId() const;
    const std::string &GetFromSectionId() const;
    const std::string &GetToSectionId() const;
    TurnLink GetTurnLink() const;

 private:
    SectionLinkData section_link_data_;
};
}  // namespace hdm

}  // namespace saffron
