#pragma once
#include <string>
#include <vector>
#include <memory>
#include <unordered_map>

#include "map/data_type/hdmap_data.hpp"
namespace saffron {
namespace hdm {
class Line {
 public:
    using ConstPtr = std::shared_ptr<const Line>;
    using Ptr = std::shared_ptr<Line>;
    using Table = std::unordered_map<std::string, Ptr>;
    using Vector = std::vector<ConstPtr>;

 public:
    Line() = default;
    explicit Line(LineData &&line_data);
    explicit Line(const LineData &line_data);
    const LineData &GetData() const;
    const std::string &GetId() const;
    const std::vector<Point3D_t> &GetGeometry() const;
    size_t GetNearestIdx(const Point3D_t &position) const;
    float64_t GetRemainLength(const Point3D_t &position) const;
    float64_t GetRemainLength(size_t index) const;
    float64_t GetHeading(const Point3D_t &position) const;
    float64_t GetHeading(size_t index) const;
    float64_t Distance2Point(const Point3D_t &position) const;
    float64_t GetWidth() const;
    bool IsVirtual() const;
    bool IsSolid() const;
    bool IsWhite() const;
    bool IsYellow() const;
    bool IsRoadSide() const;
    uint16_t GetLineType() const;

 private:
    size_t points_num_;
    LineData line_data_;
};
}  // namespace hdm
}  // namespace saffron
