#pragma once

#include "cereal/archives/binary.hpp"
#include "cereal/archives/json.hpp"
#include "cereal/archives/xml.hpp"
#include "cereal/archives/portable_binary.hpp"
#include "cereal/types/string.hpp"
#include "cereal/types/vector.hpp"
#include "map/serialization/base_serialize.hpp"

#include "map/data_type/hdmap_data.hpp"

namespace saffron {
namespace hdm {

#define REGITSTER_Lane_Type(XX)                        \
    XX(LaneType::NONE, "NONE")                         \
    XX(LaneType::DRIVING, "driving")                   \
    XX(LaneType::LEFT_WAITING, "left_waiting")         \
    XX(LaneType::RIGHT_WAITING, "right_waiting")       \
    XX(LaneType::STRAIGHT_WAITING, "straight_waiting") \
    XX(LaneType::BUS, "bus")                           \
    XX(LaneType::BICYCLE, "bicycle")                   \
    XX(LaneType::EMERGENCY, "emergency")

REGISTER_CEREAL_ENUM_SERIALIZE(LaneType, REGITSTER_Lane_Type)

#define REGITSTER_Road_Type(XX)      \
    XX(RoadType::NONE, "NONE")       \
    XX(RoadType::NORMAL, "normal")   \
    XX(RoadType::WAITING, "waiting") \
    XX(RoadType::SIDE, "side")       \
    XX(RoadType::ROUNDABOUT, "roundabout")

REGISTER_CEREAL_ENUM_SERIALIZE(RoadType, REGITSTER_Road_Type)

#define REGITSTER_Turn_Link(XX)                \
    XX(TurnLink::NONE, "NONE")                 \
    XX(TurnLink::LEFT, "left")                 \
    XX(TurnLink::RIGHT, "right")               \
    XX(TurnLink::LEFT_AROUND, "left_around")   \
    XX(TurnLink::RIGHT_AROUND, "right_around") \
    XX(TurnLink::STRAIGHT, "straight")

REGISTER_CEREAL_ENUM_SERIALIZE(TurnLink, REGITSTER_Turn_Link)

REGISTER_CEREAL_SERIALIZE(Header& header) {  // NOLINT
    CEREAL_PAIR(header, version);
    CEREAL_PAIR(header, date);
    CEREAL_PAIR(header, vendor);
    CEREAL_PAIR(header, right_hand_drive);
    CEREAL_PAIR(header, origin);
}

REGISTER_CEREAL_SERIALIZE(Boundary& boundary) {  // NOLINT
    CEREAL_PAIR(boundary, is_virtual);
    CEREAL_PAIR(boundary, geometry);
}

REGISTER_CEREAL_SERIALIZE(TrafficLightData& traffic_light_data) {  // NOLINT
    CEREAL_PAIR(traffic_light_data, id);
    CEREAL_PAIR(traffic_light_data, geometry);
    CEREAL_PAIR(traffic_light_data, type);
}

REGISTER_CEREAL_SERIALIZE(GroupTrafficLightData& data) {  // NOLINT
    CEREAL_PAIR(data, id);
    CEREAL_PAIR(data, traffic_light_ids);
    CEREAL_PAIR(data, turn_type);
}

REGISTER_CEREAL_SERIALIZE(ObjectData& object) {  // NOLINT
    CEREAL_PAIR(object, id);
    CEREAL_PAIR(object, geometry);
}

REGISTER_CEREAL_SERIALIZE(CrosswalkData& crosswalk_data) {  // NOLINT
    CEREAL_PAIR(crosswalk_data, id);
    CEREAL_PAIR(crosswalk_data, geometry);
    CEREAL_PAIR(crosswalk_data, contain_ids);
}

REGISTER_CEREAL_SERIALIZE(GuideSignData& guide_sign_data) {  // NOLINT
    CEREAL_PAIR(guide_sign_data, id);
    CEREAL_PAIR(guide_sign_data, geometry);
    CEREAL_PAIR(guide_sign_data, type);
}

REGISTER_CEREAL_SERIALIZE(JunctionData& junction_data) {  // NOLINT
    CEREAL_PAIR(junction_data, id);
    CEREAL_PAIR(junction_data, overlap_ids);
    CEREAL_PAIR(junction_data, boundaries);
}

REGISTER_CEREAL_SERIALIZE(RoadData& road_data) {  // NOLINT
    CEREAL_PAIR(road_data, id);
    CEREAL_PAIR(road_data, name);
    CEREAL_PAIR(road_data, length);
    CEREAL_PAIR(road_data, overlap_ids);
    CEREAL_PAIR(road_data, section_ids);
    CEREAL_PAIR(road_data, predecessor_ids);
    CEREAL_PAIR(road_data, successor_ids);
    CEREAL_PAIR(road_data, type);
}

REGISTER_CEREAL_SERIALIZE(SectionData& section_data) {  // NOLINT
    CEREAL_PAIR(section_data, id);
    CEREAL_PAIR(section_data, road_id);
    CEREAL_PAIR(section_data, length);
    CEREAL_PAIR(section_data, lane_ids);
    CEREAL_PAIR(section_data, overlap_ids);
    CEREAL_PAIR(section_data, left_boundary_id);
    CEREAL_PAIR(section_data, right_boundary_id);
    CEREAL_PAIR(section_data, predecessor_ids);
    CEREAL_PAIR(section_data, successor_ids);
}

REGISTER_CEREAL_SERIALIZE(NeighborLane& neighbor_lane) {  // NOLINT
    CEREAL_PAIR(neighbor_lane, lane_id);
    CEREAL_PAIR(neighbor_lane, forward);
    CEREAL_PAIR(neighbor_lane, change);
}

REGISTER_CEREAL_SERIALIZE(LaneLinkData& lane_link) {  // NOLINT
    CEREAL_PAIR(lane_link, id);
    CEREAL_PAIR(lane_link, from_lane_id);
    CEREAL_PAIR(lane_link, to_lane_id);
    CEREAL_PAIR(lane_link, ref_line_id);
    CEREAL_PAIR(lane_link, turn_link);
}

REGISTER_CEREAL_SERIALIZE(RoadLinkData& road_link) {  // NOLINT
    CEREAL_PAIR(road_link, id);
    CEREAL_PAIR(road_link, from_road_id);
    CEREAL_PAIR(road_link, to_road_id);
    CEREAL_PAIR(road_link, turn_link);
}

REGISTER_CEREAL_SERIALIZE(SectionLinkData& section_link) {  // NOLINT
    CEREAL_PAIR(section_link, id);
    CEREAL_PAIR(section_link, from_section_id);
    CEREAL_PAIR(section_link, to_section_id);
    CEREAL_PAIR(section_link, turn_link);
}

REGISTER_CEREAL_SERIALIZE(LaneData& lane_data) {  // NOLINT
    CEREAL_PAIR(lane_data, id);
    CEREAL_PAIR(lane_data, road_id);
    CEREAL_PAIR(lane_data, section_id);
    CEREAL_PAIR(lane_data, left_line_id);
    CEREAL_PAIR(lane_data, right_line_id);
    CEREAL_PAIR(lane_data, center_line_id);
    CEREAL_PAIR(lane_data, overlap_ids);
    CEREAL_PAIR(lane_data, lane_type);
    CEREAL_PAIR(lane_data, max_speed);
    CEREAL_PAIR(lane_data, min_speed);
    CEREAL_PAIR(lane_data, turn_type);
    CEREAL_PAIR(lane_data, length);
    CEREAL_PAIR(lane_data, widths);
    CEREAL_PAIR(lane_data, left_neighbor_lane);
    CEREAL_PAIR(lane_data, right_neighbor_lane);
    CEREAL_PAIR(lane_data, successor_ids);
    CEREAL_PAIR(lane_data, predecessor_ids);
}

REGISTER_CEREAL_SERIALIZE(LineData& line_data) {  // NOLINT
    CEREAL_PAIR(line_data, id);
    CEREAL_PAIR(line_data, geometry);
    CEREAL_PAIR(line_data, width);
    CEREAL_PAIR(line_data, line_type);
}
REGISTER_CEREAL_SERIALIZE(RoutingMapData& routing_map_data) {  // NOLINT
    CEREAL_PAIR(routing_map_data, lanes);
    CEREAL_PAIR(routing_map_data, junctions);
    CEREAL_PAIR(routing_map_data, roads);
    CEREAL_PAIR(routing_map_data, sections);
    CEREAL_PAIR(routing_map_data, lane_links);
    CEREAL_PAIR(routing_map_data, section_links);
    CEREAL_PAIR(routing_map_data, road_links);
    CEREAL_PAIR(routing_map_data, group_traffic_lights);
}
REGISTER_CEREAL_SERIALIZE(SemanticMapData& semantic_map_data) {  // NOLINT
    CEREAL_PAIR(semantic_map_data, lines);
    CEREAL_PAIR(semantic_map_data, guide_signs);
    CEREAL_PAIR(semantic_map_data, traffic_lights);
    CEREAL_PAIR(semantic_map_data, obstacles);
    CEREAL_PAIR(semantic_map_data, sidewalks);
    CEREAL_PAIR(semantic_map_data, stop_lines);
    CEREAL_PAIR(semantic_map_data, poles);
    CEREAL_PAIR(semantic_map_data, road_markers);
    CEREAL_PAIR(semantic_map_data, crosswalks);
    CEREAL_PAIR(semantic_map_data, park_areas);
}

REGISTER_CEREAL_SERIALIZE(HDMapData& hdmap_data) {  // NOLINT
    CEREAL_PAIR(hdmap_data, header);
    CEREAL_PAIR(hdmap_data, semantic_map_data);
    CEREAL_PAIR(hdmap_data, routing_map_data);
}

}  // namespace hdm
}  // namespace saffron
