#pragma once

#include "cereal/archives/binary.hpp"
#include "cereal/archives/json.hpp"
#include "cereal/archives/xml.hpp"
#include "cereal/archives/portable_binary.hpp"
#include "cereal/types/vector.hpp"
#include "cereal/types/string.hpp"

#include "ad_common/config_utils/macros.hpp"
#include "map/data_type/routing_data.hpp"
#include "common/serialization/base_serialize.hpp"

namespace saffron {
namespace routing {

REGISTER_CEREAL_SERIALIZE(Connection &connection) {  // NOLINT
    CEREAL_PAIR(connection, from_section);
    CEREAL_PAIR(connection, to_section);
    CEREAL_PAIR(connection, from_lane);
    CEREAL_PAIR(connection, to_lane);
}

REGISTER_CEREAL_SERIALIZE(RoutingNode &node) {  // NOLINT
    CEREAL_PAIR(node, id);
    CEREAL_PAIR(node, type);
    CEREAL_PAIR(node, lane_id);
    CEREAL_PAIR(node, road_id);
    CEREAL_PAIR(node, accu_length);
    CEREAL_PAIR(node, connection);
}
REGISTER_CEREAL_SERIALIZE(RoutingRequest &routing_request) {  // NOLINT
    CEREAL_PAIR(routing_request, departure);
    CEREAL_PAIR(routing_request, mid_points);
    CEREAL_PAIR(routing_request, black_list_points);
    CEREAL_PAIR(routing_request, mid_lane_ids);
    CEREAL_PAIR(routing_request, black_list_lanes);
    CEREAL_PAIR(routing_request, destination);
    CEREAL_PAIR(routing_request, pullover);
}

REGISTER_CEREAL_SERIALIZE(RoutingResponse &info) {  // NOLINT
    CEREAL_PAIR(info, timestamp);
    CEREAL_PAIR(info, status);
    CEREAL_PAIR(info, node_list);
    CEREAL_PAIR(info, navigation_line);
    CEREAL_PAIR(info, destination);
}

}  // namespace routing
}  // namespace saffron
